import { expect } from 'chai';
import { spy, stub } from 'sinon';
import fs from 'fs';

import compiler from './compiler';

let memoryReadFileStub;
let fsReadFileStub;

describe('compiler middleware', () => {
  describe('Running in development environment', () => {
    const config = {
      publicPath: 'foo/',
      distOutput: '/public',
      isDevEnvironment: true,
      indexFile: 'index.html',
    };

    const app = {
      locals: {
        compiler: {
          outputFileSystem: {
            readFile: () => {},
          },
          plugin: () => {},
        },
        html: null,
      },
    };
    const middleware = compiler(app, config);

    beforeEach(() => {
      memoryReadFileStub = stub(app.locals.compiler.outputFileSystem, 'readFile')
        .yields(null, '<div>Index from webpack</div>');
    });

    afterEach(() => {
      memoryReadFileStub.restore();
    });

    it('should fetch the index.html from the webpack compiler', async () => {
      const next = spy();
      const res = { set: spy() };
      await middleware({}, res, next);

      expect(app.locals.html).to.equal('<div>Index from webpack</div>');
      expect(next.called).to.equal(true);
    });

    it('should only use the cached index each subsequent request', () => {
      const next = spy();
      middleware({}, { set: spy() }, next);

      expect(app.locals.html).to.equal('<div>Index from webpack</div>');
    });
  });

  describe('Running in production environment', () => {
    const config = {
      publicPath: 'foo/',
      distOutput: '/public',
      isDevEnvironment: false,
      context: '/',
      indexFile: 'index.html',
    };
    const app = { locals: { html: null } };
    const middleware = compiler(app, config);

    beforeEach(() => {
      fsReadFileStub = stub(fs, 'readFile')
        .yields(null, '<div>Index from fs</div>');
    });

    afterEach(() => {
      fsReadFileStub.restore();
    });

    it('should fetch the index.html from the file system', async () => {
      const next = spy();
      const res = { set: spy() };
      await middleware({}, res, next);

      expect(app.locals.html).to.equal('<div>Index from fs</div>');
      expect(next.called).to.equal(true);
    });

    it('should only use the cached index each subsequent request', () => {
      const next = spy();
      middleware({}, { set: spy() }, next);

      expect(app.locals.html).to.equal('<div>Index from fs</div>');
    });
  });
});
