//  @flow
import webpack from 'webpack';
import path from 'path';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import ExtractTextPlugin from 'extract-text-webpack-plugin';
import ChunkManifestPlugin from 'chunk-manifest-webpack-plugin';
import WebpackChunkHash from 'webpack-chunk-hash';
import UglifyJSPlugin from 'uglifyjs-webpack-plugin';

import { getBundlesByName } from './../utils/bundling';
import defaultConfig from './../defaultConfig';

import type { IConfig } from '../../flow/IConfig';

const isVendor = ({ resource }) => (
  resource && resource.indexOf('node_modules') >= 0 && resource.match(/\.js$/)
);

export default ({
  context = defaultConfig.context,
  distOutput = defaultConfig.distOutput,
  jsPath = defaultConfig.jsPath,
  cssPath = defaultConfig.cssPath,
  devCssBundle = defaultConfig.devCssBundle,
  prodCssBundle = defaultConfig.prodCssBundle,
  isDevEnvironment,
  plugins = [],
  routes,
}: IConfig) => {
  //  Gets all { bundle } values from the routeConfig
  const bundleNames = getBundlesByName(routes);

  //  Environment plugins define how webpack should
  //  be bundled for both production and development.
  //  Things like hot-module-reloading aren't required for
  //  prod, as an example.
  const environmentPlugins = (!isDevEnvironment)
    ? [
      new webpack.HashedModuleIdsPlugin(),
      new WebpackChunkHash(),
      new ChunkManifestPlugin({
        filename: `${jsPath}/chunk-manifest.js`,
        manifestVariable: 'webpackManifest',
        inlineManifest: true,
      }),
      new UglifyJSPlugin(),
    ] : [
      new webpack.HotModuleReplacementPlugin(),
    ];

  return [
    //  Assign any custom
    //  plugins from config
    ...plugins,

    //  Implement all environment
    //  specific plugins (i.e. chunkhashing)
    ...environmentPlugins,

    //  Webpack bundle injection
    //  An array of chunks to be excluded from
    //  the HtmlWebpackPlugin injection.  This is important
    //  because routes that require lazy loading will be
    //  injected into the <head> when they are needed
    new HtmlWebpackPlugin({
      inject: true,
      template: `${context}/index.html`,
      filename: path.join(process.cwd(), '.', distOutput, 'index.html'),
      excludeChunks: bundleNames,
    }),

    //  Chunk common vendor / main files
    new webpack.optimize.CommonsChunkPlugin({
      name: ['vendor', 'manifest'],
      minChunks: isVendor,
    }),

    //  Adds a CommonsChunkPlugin for each
    ...(bundleNames.length) && bundleNames.map(name => (
      new webpack.optimize.CommonsChunkPlugin({
        name,
        chunks: [name],
        minChunks: 1,
      })
    )),

    new ExtractTextPlugin({
      filename: (isDevEnvironment)
        ? devCssBundle
        : `${cssPath}/${prodCssBundle}`,
      allChunks: true,
    }),

    //  Output bundles will run in
    //  either production or development
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: JSON.stringify(isDevEnvironment ? 'development' : 'production'),
        API_URL: JSON.stringify('http://10.10.10.10:8080/bands'),
      },
    }),
  ];
};
